#!/usr/bin/env python3.5 

'''
This work is licensed under the Creative Commons Attribution-ShareAlike 4.0 International License. To view a copy of this license, visit http://creativecommons.org/licenses/by-sa/4.0/.

Website: http://projects.cl.uni-heidelberg.de/SR3de/
Contact: Eva Mujdricza-Maydt, mujdricza@cl.uni-heidelberg.de

'''
__version__ = "20170407"

'''
Inputs:
- inputs/SR3deVN+SALSA+CoNLL2009_mappings.csv (included)
- SR3de_VN_annotation-files (included)
- original SALSA xml-files for verbal predicates [http://www.coli.uni-saarland.de/projects/salsa/]
- original conll-formatted files for the CoNLL 2009 ST [https://ufal.mff.cuni.cz/conll2009-st/]

Outputs:
- generated folder structure with parallel data

Generated folder structure:
+ SR3de
  +- FN
     +- sr3de_dev.conll
     +- sr3de_test.conll
     +- sr3de_train.conll
  +- PB
     +- sr3de_dev.conll
     +- sr3de_test.conll
     +- sr3de_train.conll
  +- VN
     +- sr3de_dev.conll
     +- sr3de_test.conll
     +- sr3de_train.conll

'''

import os
import sys
import readers.CoNLL2009reader as CoNLL2009reader
import readers.SALSAreader as SALSAreader
import readers.utils_SALSA as utils_SALSA
import readers.utils_conversion as utils
import readers.data_converter as converter
import argparse


def generate_SR3de_VN(input_path_sr3de_onlySRL, input_path_conll2009, parallel_entry_list, output_path):
    
    parallel_VN2entry_dict = utils.get_parallel_entry_dict(parallel_entry_list, key=utils.DATA_NAME.SR3de_VN.value)
    
    sr3de_VN_onlySRL_data_dict = CoNLL2009reader.read_conllformatted_data_per_dataset_dict(input_path_sr3de_onlySRL, utils.SR3de_VN_FILES, min_conll_column_index=1) # NOTE min_index=1
    
    conll2009_data_dict = CoNLL2009reader.read_conllformatted_data_per_dataset_dict(input_path_conll2009, utils.CoNLL2009_FILES) #no additional min_conll_column_index restriction
    
    fully_annotated_data_dict = {}
    for dataset, srlanno_sr3devn in sr3de_VN_onlySRL_data_dict.items():
        parallel_VN2entries_for_dataset_dict = parallel_VN2entry_dict[dataset]
        #print("DATASET = {} with parallel_entries_for_dataset_dict = {}".format(dataset, len(parallel_entries_for_dataset_dict)))
        
        fully_annotated_sentence_list = converter.get_fully_annotated_VN_sentence_list(srlanno_sr3devn, parallel_VN2entries_for_dataset_dict, conll2009_data_dict)
        fully_annotated_data_dict[dataset] = fully_annotated_sentence_list
        
        #write
        output_file_name = utils.normalize_path(output_path) + utils.SR3de_PARALLEL_FILES.__members__[dataset].value
        converter.write_file_with_extracted_annotation(output_file_name, fully_annotated_sentence_list)
    
    return fully_annotated_data_dict



def generate_SR3de_PB(input_path_sr3de_onlySRL, input_path_conll2009, parallel_entry_list, output_path_for_PB):
    parallel_VN2entry_dict = utils.get_parallel_entry_dict(parallel_entry_list, key=utils.DATA_NAME.SR3de_VN.value)
    
    sr3de_VN_onlySRL_data_dict = CoNLL2009reader.read_conllformatted_data_per_dataset_dict(input_path_sr3de_onlySRL, utils.SR3de_VN_FILES, min_conll_column_index=1) # NOTE min_index=1
    
    conll2009_data_dict = CoNLL2009reader.read_conllformatted_data_per_dataset_dict(input_path_conll2009, utils.CoNLL2009_FILES) #no additional min_conll_column_index restriction
    
    fully_annotated_data_dict = {}
    for dataset, srlanno_sr3devn in sr3de_VN_onlySRL_data_dict.items():
        parallel_VN2entries_for_dataset_dict = parallel_VN2entry_dict[dataset]
        #print("DATASET = {} with parallel_entries_for_dataset_dict = {}".format(dataset, len(parallel_entries_for_dataset_dict)))
        fully_annotated_VN_sentence_list = converter.get_fully_annotated_VN_sentence_list(srlanno_sr3devn, parallel_VN2entries_for_dataset_dict, conll2009_data_dict)
        
        fully_annotated_PB_sentence_list = converter.get_fully_annotated_PB_sentence_list(fully_annotated_VN_sentence_list, parallel_VN2entries_for_dataset_dict, conll2009_data_dict)
        fully_annotated_data_dict[dataset] = fully_annotated_PB_sentence_list

        # write
        output_file_name = utils.normalize_path(output_path_for_PB) + utils.SR3de_PARALLEL_FILES.__members__[dataset].value
        converter.write_file_with_extracted_annotation(output_file_name, fully_annotated_PB_sentence_list)
    
    return fully_annotated_data_dict


def generate_SR3de_FN(input_path_sr3de_onlySRL, input_path_conll2009, input_path_salsa, parallel_entry_list, output_path):
    
    parallel_VN2entry_dict = utils.get_parallel_entry_dict(parallel_entry_list, key=utils.DATA_NAME.SR3de_VN.value)
    
    sr3de_VN_onlySRL_data_dict = CoNLL2009reader.read_conllformatted_data_per_dataset_dict(input_path_sr3de_onlySRL, utils.SR3de_VN_FILES, min_conll_column_index=1) # NOTE min_index=1
    
    conll2009_data_dict = CoNLL2009reader.read_conllformatted_data_per_dataset_dict(input_path_conll2009, utils.CoNLL2009_FILES) #no additional min_conll_column_index restriction
    
    sentenceid_to_frames_dict = SALSAreader.read_salsa_data_per_sentence_dict(input_path_salsa, parallel_entry_list)
    
    fully_annotated_data_dict = {}
    for dataset, srlanno_sr3devn in sr3de_VN_onlySRL_data_dict.items():
        parallel_VN2entries_for_dataset_dict = parallel_VN2entry_dict[dataset]
        #print("DATASET = {} with parallel_entries_for_dataset_dict = {}".format(dataset, len(parallel_entries_for_dataset_dict)))
        
        fully_annotated_VN_sentence_list = converter.get_fully_annotated_VN_sentence_list(srlanno_sr3devn, parallel_VN2entries_for_dataset_dict, conll2009_data_dict)
        
        fully_annotated_FN_sentence_list = converter.get_fully_annotated_FN_sentence_list(sentenceid_to_frames_dict, parallel_VN2entries_for_dataset_dict, fully_annotated_VN_sentence_list)
        fully_annotated_data_dict[dataset] = fully_annotated_FN_sentence_list

        # write
        output_file_name = utils.normalize_path(output_path) + utils.SR3de_PARALLEL_FILES.__members__[dataset].value
        converter.write_file_with_extracted_annotation(output_file_name, fully_annotated_FN_sentence_list)
    
    return fully_annotated_data_dict

def load_argument_parser():
    parser = argparse.ArgumentParser(
        description='creates the SR3de "Semantic Role Triple German Dataset"')
    parser.add_argument(
        '-pb', dest="path_to_pb", type=str, required=True,
        help='path to CoNLL2009 German annotation')
    parser.add_argument(
        '-fn', dest="path_to_fn", type=str, required=True,
        help='path to SALSA main folder')
    parser.add_argument(
        '-vn', dest="path_to_vn", type=str, default="inputs/SR3de_VN-onlySR/",
        help='path to the VN-style annotation part (optional)')
    parser.add_argument(
        '-o', dest="output_path", type=str, required=True,
        help='output path for the parallel annotation data')
   
    return parser

def main_create_parallel_data():
    '''
STATISTICS on extracted predicate argument structures in generated VN (SR3de_VN) data:
- dataset 'train' entails 2123 sentences with 2189 annotated predicates and 4296 arguments.
- dataset 'dev' entails 263 sentences with 275 annotated predicates and 538 arguments.
- dataset 'test' entails 482 sentences with 520 annotated predicates and 1025 arguments.

STATISTICS on extracted predicate argument structures in generated PB (CoNLL2009) data:
- dataset 'train' entails 2123 sentences with 2189 annotated predicates and 4282 arguments.
- dataset 'dev' entails 263 sentences with 275 annotated predicates and 527 arguments.
- dataset 'test' entails 482 sentences with 520 annotated predicates and 1022 arguments.

STATISTICS on extracted predicate argument structures in generated FN (SALSA) data:
- dataset 'train' entails 2123 sentences with 2189 annotated predicates and 4236 arguments.
- dataset 'dev' entails 263 sentences with 275 annotated predicates and 527 arguments.
- dataset 'test' entails 482 sentences with 520 annotated predicates and 1008 arguments.

    '''
    '''
    args = sys.argv
    if len(args) == 2 and args[1] == "hard":
        input_path_sr3deVN_onlySRL = "inputs/SR3de_VN-onlySR/"
        input_path_conll2009 = "/Users/eva/my/uni/CLARIN-D/KP3/KP3B/Data/rawData/CoNLL2009-ST/"
        input_path_salsa = "/Users/eva/my/uni/Data/SALSA/" # salsa-corpora-2.0/salsa-corpora
        output_path = "/Users/eva/my/uni/CLARIN-D/KP3/KP3B/Data/Outputs/"
    
    '''
    argparser = load_argument_parser()
    args = argparser.parse_args()

    input_path_conll2009 = utils.normalize_path(args.path_to_pb)
    input_path_sr3deVN_onlySRL = utils.normalize_path(args.path_to_vn)
    input_path_salsa = utils.normalize_path(args.path_to_fn)
    output_path = utils.normalize_path(args.output_path)
    
    utils.create_path_if_needed(output_path)
    utils.create_parallel_path_structure(output_path)
    print("OUTPUT PATH for the SR3de parallel corpus: " + output_path)
        
    mapping_file_name = "inputs/SR3deVN+SALSA+CoNLL2009_mappings.csv"
    parallel_entry_list = utils.read_mapping_file(mapping_file_name)
    
    sr3de_VN_data_dict = generate_SR3de_VN(input_path_sr3deVN_onlySRL, input_path_conll2009, parallel_entry_list, output_path + utils.OUTPUT_PATH_PART.SR3de_VN.value)
    CoNLL2009reader.print_pas_statistics("generated VN (SR3de_VN) data", sr3de_VN_data_dict)
    
    sr3de_PB_data_dict = generate_SR3de_PB(input_path_sr3deVN_onlySRL, input_path_conll2009, parallel_entry_list, output_path + utils.OUTPUT_PATH_PART.SR3de_PB.value)
    CoNLL2009reader.print_pas_statistics("generated PB (CoNLL2009) data", sr3de_PB_data_dict)
    
    sr3de_FN_data_dict = generate_SR3de_FN(input_path_sr3deVN_onlySRL, input_path_conll2009, input_path_salsa, parallel_entry_list, output_path + utils.OUTPUT_PATH_PART.SR3de_FN.value)
    CoNLL2009reader.print_pas_statistics("generated FN (SALSA) data", sr3de_FN_data_dict)

if __name__ == "__main__":
    main_create_parallel_data()
    