#!/usr/bin/env python3.5 

'''
This work is licensed under the Creative Commons Attribution-ShareAlike 4.0 International License. To view a copy of this license, visit http://creativecommons.org/licenses/by-sa/4.0/.

Website: http://projects.cl.uni-heidelberg.de/SR3de/
Contact: Eva Mujdricza-Maydt, mujdricza@cl.uni-heidelberg.de

'''

__author__ = "mujdricza-maydt"


'''
Reads CoNLL2009 formatted files and extracts in particular the predicate argument structures (pas) in these files.

'''

from enum import Enum
from readers import utils_conversion as utils


class CONLL2009_ANNOIDX(Enum):
    '''Indices in the CoNLL2009 columns'''
    ID = 0 #word number starting with 1
    FORM = 1 #token
    LEMMA = 2
    PLEMMA = 3
    POS = 4
    PPOS = 5
    FEAT = 6 #morphological information
    PFEAT = 7
    HEAD = 8 #dependence
    PHEAD = 9
    DEPREL = 10
    PDEPREL = 11
    FILLPRED = 12 #Y or _
    PRED = 13 #if FILLPRED=="Y"
    APRED1 = 14 # and above, if FILLPRED=="Y"
    #NOTE that there is no coreference annotation in the files, and it is also assumed not to have this annotation level!

class FORMAT(Enum):
    '''variables and constants commonly used'''
    NOVALUE = "_"
    FILLEDPRED = "Y"
    TAB = "\t"
    NL = "\n"


def read_conll_formatted_sentences(input_file_name, min_index=CONLL2009_ANNOIDX.FILLPRED.value):
    '''
    reads the CoNLL2009 file in a simple list format
    :param input_file_name: name of a file in CoNLL2009 ST format
    :param min_index: indicates how many columns a well-formed line of a sentence should have at least 
    :return: list of sentences where a sentence is a list of lines representing a token in the sentence; a line is a list of the columns (annotation layers for a token) 
    '''
    
    list_of_sentences = []
    
    current_sentence = []
    with open(input_file_name) as f:
        for line in f.readlines():
            line = line.strip()
            
            if len(line) == 0:
                if len(current_sentence) > 0:
                    list_of_sentences.append(current_sentence)
                    current_sentence = []
                continue
            
            splitted_line = line.split()  
            if not len(splitted_line) > min_index: 
                raise IndexError("Current line has a non-expected format: expected columns:{} > available: {}; see line:\n'{}'".format(min_index+1, len(splitted_line), line))
            current_sentence.append(splitted_line)
    
    if len(current_sentence) > 0: #if there is a last sentence without a new line after it
        list_of_sentences.append(current_sentence)
        current_sentence = []
    
    return list_of_sentences


def get_pas_annotation_list(current_sentence):
    '''
    extracts the predicate argument annotation for the current sentence
    :param current_sentence: a CoNLL2009 formatted sentence as a list of its tokens (see read_conll_formatted_sentences())
    :return: list of the predicate argument structure annotations
    '''
    #[{(pred_idx, label): [list_of_args_with_labels]}, {}]
    pas_dict_list = []
    
    predicates = __get_predicates(current_sentence)
    #print("predicates: ", predicates)
    for predicate_number_idx in range(len(predicates)): #(pred_tok_idx, pred_label)
        current_pas_dict = {}
        arguments = __get_arguments(current_sentence, predicate_number_idx)
        current_pas_dict[predicates[predicate_number_idx]] = arguments
        pas_dict_list.append(current_pas_dict)
    return pas_dict_list


def __get_predicates(list_of_token_infos_for_sentence):
    
    predicates = []
    
    for token_idx in range(len(list_of_token_infos_for_sentence)):
        token_line_split = list_of_token_infos_for_sentence[token_idx]
        if len(token_line_split) <= CONLL2009_ANNOIDX.FILLPRED.value:
            raise IndexError("NOT ASSUMED SHORT LINE with length={}, token_idx={}, all_lines_in_sent:\n{}".format(len(token_line_split), token_idx, str(list_of_token_infos_for_sentence)))
        if token_line_split[CONLL2009_ANNOIDX.FILLPRED.value] != FORMAT.NOVALUE.value:
            predicate_label = token_line_split[CONLL2009_ANNOIDX.PRED.value]
            predicates.append((token_idx, predicate_label))
    
    return predicates

def __get_arguments(list_of_token_infos_for_sentence, predicate_number_index):
    
    arguments = []
    
    column_idx = CONLL2009_ANNOIDX.APRED1.value + predicate_number_index
    for token_idx in range(len(list_of_token_infos_for_sentence)):
        if len(list_of_token_infos_for_sentence[token_idx]) <= column_idx:
            raise IndexError("NOT ASSUMED SHORT LINE with token_idx={} searching for arguments of the {}. predicate in column idx={}, sentence lines:\n{}\n".format(token_idx, predicate_number_index, column_idx, "\n".join([str(x) for x in list_of_token_infos_for_sentence])))
        arg_label = list_of_token_infos_for_sentence[token_idx][column_idx]
        if arg_label == FORMAT.NOVALUE.value:
            pass
        else:
            arg = (token_idx, arg_label)
            arguments.append(arg)
    
    return arguments



def print_pas_statistics(data_info, data_dict_with_conll_formatted_sentences_for_all_datasets):
    
    msg = "STATISTICS on extracted predicate argument structures in {}:\n".format(data_info)
    for dataset, conll_formatted_sentence_list in data_dict_with_conll_formatted_sentences_for_all_datasets.items():
        sentences, predicates, arguments = __count_pas_annotations(conll_formatted_sentence_list)
        msg_d = "- dataset '{}' entails {} sentences with {} annotated predicates and {} arguments.\n".format(dataset, sentences, predicates, arguments)
        msg += msg_d
    print(msg)
    
    
def __count_pas_annotations(conll_formatted_sentences):
    #[{(pred_idx, label): [list_of_args_with_labels]}, {}]
    
    sentences = len(conll_formatted_sentences)
    predicates = 0
    arguments = 0
    
    for sentence in conll_formatted_sentences:
        pas_dict_list = get_pas_annotation_list(sentence)
        
        for pas_dict in pas_dict_list:
            predicates += 1
            for pred_info, arg_list in pas_dict.items():
                arguments += len(arg_list)
            
    return ( sentences, predicates, arguments )

def read_conllformatted_data_per_dataset_dict(input_path, enum_class_with_dataset_filenames, min_conll_column_index=CONLL2009_ANNOIDX.FILLPRED.value):
    data_dict = {}
    for dataset in list(utils.DATASET.__members__):
        input_file_for_dataset = enum_class_with_dataset_filenames.__members__[dataset].value
        data_sentences = read_conll_formatted_sentences(input_path + input_file_for_dataset, min_index = min_conll_column_index) # NOTE min_index=1
        data_dict[dataset] = data_sentences
    return data_dict
