#!/usr/bin/env python3.5 

'''
This work is licensed under the Creative Commons Attribution-ShareAlike 4.0 International License. To view a copy of this license, visit http://creativecommons.org/licenses/by-sa/4.0/.

Website: http://projects.cl.uni-heidelberg.de/SR3de/
Contact: Eva Mujdricza-Maydt, mujdricza@cl.uni-heidelberg.de

'''

__author__ = "mujdricza-maydt"

'''
Reads the SALSA-2.0 sentences into a conll-formatted structure.

Steps
- choose only sentences with needed sentence id,
- make the annotation according the conll head information (read from CoNLL2009 data)
- aggregate annotations for a sentence if more than one PAS structure is given

'''

from readers import utils_SALSA
from readers import utils_conversion

def get_xml_part_for_sentences(salsa_xml_filename, salsa_sentence_idnumbers = None):
    '''
    reads the sentences with s tag 
    :param salsa_xml_filename: a SALSA xml file name
    :param salsa_sentence_idnumbers: if given, only the sentences with the specified id numbers will be extracted
    :return: dictionary of relevant sentences with the sentence id number as key and the list of the sentence lines as value
    '''
    
    sentence_lines_dict = {}
    
    f = open(salsa_xml_filename)
    lines = []
    for line in f.readlines():
        lines.append(line.strip()) #strip the lines
    f.close()
    
    current_list_of_lines = []
    j = 0 #for the subroutine
    for i in range(len(lines)):
        
        if i < j: #do not visit already visited lines
            continue
        
        line = lines[i]
        if line.startswith(utils_SALSA.STRING.SENTENCE_OPEN_start.value):  # "<s id"
            match = utils_SALSA.PATTERN.ATT_S_id.value.search(line)
            if not match:
                raise NameError("NOT ASSUMED SENTENCE OPENING TAG: {}".format(line))
            current_sentid = int(match.group(utils_SALSA.GROUPNAME.sentId.value))
            if salsa_sentence_idnumbers is not None:
                if current_sentid not in salsa_sentence_idnumbers:
                    continue
                #else:
                #   print("(not needed sentence id={}".format(current_sentid))
            
            #subroutine for collecting the sentence lines
            j = i
            nextline = lines[j]
            while nextline != utils_SALSA.STRING.SENTENCE_CLOSE.value:  # "</s>"
                current_list_of_lines.append(nextline)
                #update nexline
                j += 1
                nextline = lines[j]
            
            current_list_of_lines.append(nextline) #closing </s>
            sentence_lines_dict[current_sentid] = current_list_of_lines
            #print(" -> current_list_of_lines: {} lines for sent {}".format(len(current_list_of_lines), current_sentid))
            
            j += 1
            current_list_of_lines = []
        
    #print("--> read {} sentences".format(len(sentence_lines_dict)))
    return sentence_lines_dict


def __read_tnt_graph(xml_lines_of_sentence):
    '''
    reads the terminal and nonterminal nodes 
    :param xml_lines_of_sentence: xml lines for the current sentence
    :return: tuple with terminal and nonterminal information
    '''
    terminal_dict = __get_terminals(xml_lines_of_sentence)
    nonterminal_dict = __get_nonterminals(xml_lines_of_sentence)
    nonterminal_to_terminal_scope_dict = __get_terminal_scope_for_nonterminal_dicts(terminal_dict, nonterminal_dict)

    return (terminal_dict, nonterminal_to_terminal_scope_dict)

def __get_terminals(xml_lines_of_sentence):
    terminals_dict = {}
    for i in range(len(xml_lines_of_sentence)):
        line = xml_lines_of_sentence[i]
        if line == utils_SALSA.STRING.TERMINALS_OPEN.value:
            #from the next line onwards
            j = i+1
            nextline = xml_lines_of_sentence[j]
            while nextline != utils_SALSA.STRING.TERMINALS_CLOSE.value:
                if not nextline.startswith(utils_SALSA.STRING.TERMINAL_EMPTY_start.value):
                    #e.g. <t id="s6117_17" lemma="wenn" person="--" degree="--" tense="--" mood="--" gender="--" number="--" word="wenn" case="--" morph="--" pos="KOUS">
                    #         <secedge label="CP" idref="s6117_510"/>
                    #     </t>
                    if nextline == utils_SALSA.STRING.TERMINAL_CLOSE.value \
                        or nextline.startswith(utils_SALSA.STRING.SECEDGE_EMPTY_start.value):
                        j += 1
                        nextline = xml_lines_of_sentence[j]
                        continue
                    
                    raise NameError("NOT ASSUMED TERMINAL-LINE: {}".format(nextline))
    
                token_id = None
                match = utils_SALSA.PATTERN.ATT_T_id.value.search(nextline)
                if not match:
                    raise NameError("NOT FOUND TERMINAL-id in: {}".format(nextline))
                token_id = int(match.group(utils_SALSA.GROUPNAME.tokenId.name))
    
                token_word = None
                match = utils_SALSA.PATTERN.ATT_word.value.search(nextline)
                if not match:
                    raise NameError("NOT FOUND TERMINAL-word in: {}".format(nextline))
                token_word = match.group(utils_SALSA.GROUPNAME.word.name)
                
                token_pos = None
                match = utils_SALSA.PATTERN.ATT_pos.value.search(nextline)
                if not match:
                    raise NameError("NOT FOUND TERMINAL-pos in: {}".format(nextline))
                token_pos = match.group(utils_SALSA.GROUPNAME.pos.name)
                
                terminal = (token_id, token_word, token_pos)
                terminals_dict[token_id] = terminal
                
                #update nexline
                j += 1
                nextline = xml_lines_of_sentence[j]
            break # afterwards, no terminals more assumed
    return terminals_dict


def __get_nonterminals(xml_lines_of_sentence):
    
    nonterminals_dict = {}
    for i in range(len(xml_lines_of_sentence)):
        line = xml_lines_of_sentence[i]
        if line == utils_SALSA.STRING.NONTERMINALS_OPEN.value:
            # from next line onwards
            j = i + 1
            nextline = xml_lines_of_sentence[j]
            while nextline != utils_SALSA.STRING.NONTERMINALS_CLOSE.value:
                
                #<nt id="s17157_505" cat="S">
                #    <edge label="MO" idref="s17157_11"/>
                #    <edge label="MO" idref="s17157_501"/>
                #</nt>
                if nextline.startswith(utils_SALSA.STRING.NONTERMINAL_OPEN_start.value):
                    nonterminal_id, nonterminal_dependents = __get_nonterminal(xml_lines_of_sentence, j)
                    nonterminals_dict[nonterminal_id] = nonterminal_dependents
                    #update j at least by the number of edges
                    j += len(nonterminal_dependents)+1
                
                #update nexline
                j += 1
                nextline = xml_lines_of_sentence[j]
                
            break # afterwards, no nonterminals assumed
    return nonterminals_dict

def __get_nonterminal(xml_lines_of_sentence, index):
    
    nonterminal_id = None
    nonterminal_dependents = []
    
    for i in range(index, len(xml_lines_of_sentence), 1):
        line = xml_lines_of_sentence[i]
        
        if line == utils_SALSA.STRING.NONTERMINAL_CLOSE.value:
            if nonterminal_id is None or len(nonterminal_dependents) == 0:
                raise NameError("NOT FOUND INFORMATION ON NONTERMINAL in the range of lines with index {}-{} -- corresponding lines: \n{}".format(index, i, xml_lines_of_sentence[index:i+1]))
            break
        
        elif line.startswith(utils_SALSA.STRING.NONTERMINAL_OPEN_start.value):
            match = utils_SALSA.PATTERN.ATT_NT_id.value.search(line)
            if not match: #for VROOT id --> set it to a big, non-used number like 1000
                match = utils_SALSA.PATTERN.ATT_NT_id_VROOT.value.search(line)
                nonterminal_id = int(utils_SALSA.ESCAPE.VROOT_id.value)
                if not match:
                    raise NameError("NOT FOUND NONTERMINAL-id in: {}".format(line))
            else:   
                nonterminal_id = int(match.group(utils_SALSA.GROUPNAME.ntId.value))
        
        elif line.startswith(utils_SALSA.STRING.EDGE_EMPTY_start.value):
            match = utils_SALSA.PATTERN.ATT_EDGE_idref.value.search(line)
            if not match:
                raise NameError("NOT FOUND EDGE-idref in: {}".format(line))
            nonterminal_dependent_idref = int(match.group(utils_SALSA.GROUPNAME.ntedgeIdref.value))
            nonterminal_dependents.append(nonterminal_dependent_idref)  
    
    return (nonterminal_id, nonterminal_dependents)

def __get_terminal_scope_for_nonterminal_dicts(terminal_dict, nonterminal_dict):
    '''
    terminal_dict: { t_id : (t_id, t_word, t_pos), t_id: (...), ...}
    nonterminal_dict : { nt_id : [idref, idref, ...], nt_id: [idref, idref], ...  }
    
    :param terminal_dict: 
    :param nonterminal_dict: 
    :return: 
    '''
    
    nt_to_t_ids = {}
    for nt_id, nt_dependent_idrefs in nonterminal_dict.items():
    
        current_t_ids = set()
        for nt_idref in nt_dependent_idrefs:
            current_t_ids = __get_terminal_ids_for_nt_idref(nt_idref, nt_to_t_ids, terminal_dict, current_t_ids)
    
        #sort the integer values and save
        current_t_ids = sorted(list(current_t_ids))
        
        #check
        for id in current_t_ids:
            if id not in terminal_dict.keys():
                raise NameError("NOT APPROPRIATE T-id '{}' in the list: {}".format(id, current_t_ids))
        
        nt_to_t_ids[nt_id] = current_t_ids
    
    return nt_to_t_ids

def __get_terminal_ids_for_nt_idref(nt_idref, nt_to_t_ids_dict, terminal_dict, found_t_ids):
    
    current_idref = nt_idref
    
    if current_idref in terminal_dict.keys():
        found_t_ids.add(current_idref)
    else:
        dependent_idrefs = nt_to_t_ids_dict[current_idref]
        for dependent_idref in dependent_idrefs:
            found_t_ids = __get_terminal_ids_for_nt_idref(dependent_idref, nt_to_t_ids_dict, terminal_dict, found_t_ids)
        
    return found_t_ids


def get_frames_for_sentence_terminalized_nonoverlapping(xml_lines_of_sentence, curr_sentenceid):
    
    terminals_dict, nonterminals_dict = __read_tnt_graph(xml_lines_of_sentence)
   
    
    #frame_dict:
    # { target_id : [
    #       { target_name : [(sentid, [list of tids]), (sentid, [list of tids]), ...] }
    #       { fe_name : [(sentid, [list of tids]), ...], fe_name : [...], ... }
    # ]
    #   }
    
    frames_dict = {}
    j = 0
    for i in range(len(xml_lines_of_sentence)):
        if i < j:
            continue
        
        line = xml_lines_of_sentence[i]
        #print("get_frames_for_sentence: i={}, j={}, line={}".format(i,j,line))
        if line == utils_SALSA.STRING.FRAMES_OPEN.value:
            # from the next line onwards
            j = i + 1
            nextline = xml_lines_of_sentence[j]
            while nextline != utils_SALSA.STRING.FRAMES_CLOSE.value and j < len(xml_lines_of_sentence):
                
                #e.g. <frame name="Questioning" id="abfragen_s17157_f0">
                #    <target lemma="abfragen">
                #        <fenode idref="s17157_1"/>
                #        <fenode idref="s17157_19"/>
                #    </target>
                #    <fe name="Speaker" id="abfragen_s17157_f0_e1">
                #        <fenode idref="s17157_2"/>
                #    </fe>
                #    <fe name="Addressee" id="abfragen_s17157_f0_e2">
                #        <fenode idref="s17157_3"/>
                #    </fe>
                #    <fe name="Topic" id="abfragen_s17157_f0_e3">
                #        <fenode idref="s17157_509"/>
                #    </fe>
                #</frame>
                if nextline.startswith(utils_SALSA.STRING.FRAME_OPEN_start.value):
                    #print("get frame with j={}".format(j))
                    #whole_frame_id, frame_content (<-- dict for predicate, dict for arguments )
                    whole_frame_id, frame_dict_content = _get_frame(xml_lines_of_sentence, j)
                    predicate_dict, arguments_dict = frame_dict_content
                    
                    #all id references go to terminal ids
                    predicate_dict = __replace_nt_by_t(predicate_dict, terminals_dict, nonterminals_dict, curr_sentenceid)
                    arguments_dict = __replace_nt_by_t(arguments_dict, terminals_dict, nonterminals_dict, curr_sentenceid)
                    
                    #avoid overlapping sequences
                    arguments_dict = __reduce_overlapping_t_sequences(arguments_dict)
                    
                    frames_dict[whole_frame_id] = [ predicate_dict, arguments_dict ]
                    #print("-> frame_dict_key:{}".format(frame_dict_key))
                #update nexline
                j += 1
                nextline = xml_lines_of_sentence[j]
            break # afterwards, no nonterminals assumed
    return frames_dict

def __replace_nt_by_t(label_to_position_dict, terminals_dict, nonterminals_dict, curr_sentenceid):
    '''
    
    :param label_to_position_dict: { label : [ (sentid, tntid), (...), ...] }
    :param terminals_dict: { token_id: (token_id, token_word, token_pos), ... }
    :param nonterminals_dict: e.g. { ntid: [tid, tid, ...], ...}
    :param curr_sentenceid: id of the current sentence -- if a terminal or nonterminal idref refers to a tnt beyond the sentence, an empty list will be created for it

    :return: { label : [ (sentid, [tid_list]), (...), ...] }
    '''
    
    for label in label_to_position_dict:
        
        list_of_sentid_tntid_tuples = label_to_position_dict[label]
        for i in range(len(list_of_sentid_tntid_tuples)):
            sent_id, tnt_id = list_of_sentid_tntid_tuples[i]
            
            if curr_sentenceid != sent_id: # tnt is beyond the sentence which will be NOT handled now
                label_to_position_dict[label][i] = ( sent_id, [] ) #empty list !!
            
            elif tnt_id in terminals_dict.keys():
                label_to_position_dict[label][i] = ( sent_id, [tnt_id] )
            else:
                if tnt_id in nonterminals_dict.keys():
                    t_id_list = nonterminals_dict[tnt_id]
                    label_to_position_dict[label][i] = ( sent_id, t_id_list )
                else:
                    raise NameError("NOT FOUND tnt-id: {} in extracted dicts:\n- terminals: {}\n- nonterminals: {}".format(tnt_id, terminals_dict, nonterminals_dict))
            
    return label_to_position_dict


def __reduce_overlapping_t_sequences( label_to_position_dict):
    #print("\t reducing: ", label_to_position_dict)
    #input { label : [ (sentid, [tid_list]), (...), ...] }
    
    #REDUCED ARG DICT:
    #-before: {'Entity': [(32060, [23, 24])], 'Function': [(32060, [17, 18, 19, 20, 22, 23, 24, 25])]}
    #-after: {'Entity': [(32060, [23, 24])], 'Function': [(32060, [17, 18, 19, 20, 22, 25])]} <--- overlapping tok idx deleted
    #REDUCED ARG DICT:
    #-before: {'Speaker': [(18286, [12])], 'Victim': [(18286, [14]), (18286, [13])]}
    #-after: {'Speaker': [(18286, [12])], 'Victim': [(18286, [13, 14])]} <--- partitioned sequences aggregated in one item

    
    label_list_sorted = sorted(list(label_to_position_dict.keys()))
    
    for i in range(len(label_list_sorted)):
        i_label = label_list_sorted[i]
        i_positions = label_to_position_dict[i_label]
        i_sentid, merged_i_toks = __merge_tokposition_lists(i_positions)
        label_to_position_dict[i_label] = [ (i_sentid, merged_i_toks) ]
       
        for j in range(len(label_list_sorted)):
            if i == j:
                continue
            j_label = label_list_sorted[j]
            j_positions = label_to_position_dict[j_label]
            j_sentid, merged_j_toks = __merge_tokposition_lists(j_positions)
            
            if len(merged_i_toks) > len(merged_j_toks):
                #print("\t\t i: {} > j: {}".format(merged_i_toks, merged_j_toks))
                merged_i_toks = [x for x in merged_i_toks if x not in merged_j_toks]
                #print("\t\t\t-> i: {}".format(merged_i_toks))
            label_to_position_dict[i_label] = [ (i_sentid, merged_i_toks) ]
            
            
    #print("\t ->reduced: ", label_to_position_dict)        
    return label_to_position_dict

def __merge_tokposition_lists(tok_position_list):
    
    merged_tokid_set = set()
        
    sentid = None
    for curr_sentid, curr_tokid_list in tok_position_list:
        if len(curr_tokid_list) == 0:
            continue
        sentid = curr_sentid
        merged_tokid_set = merged_tokid_set.union(set(curr_tokid_list))
    merged_tokid_list = sorted(list(merged_tokid_set))
    
    return (sentid, merged_tokid_list)


def _get_frame(xml_lines_of_sentence, index):
    
    whole_frame_id = None
    frame_name = None
    frame_content = []
    
    target_content = []
    fe_dict = {}
    
    is_target_open = False
    is_fe_open = False
    current_fe_name = None
    
    for i in range(index, len(xml_lines_of_sentence), 1):
        line = xml_lines_of_sentence[i]
        
        if line == utils_SALSA.STRING.FRAME_CLOSE.value:
            if whole_frame_id is None or frame_name is None or len(target_content) == 0: #NOTE that fe_dict could be empty
                raise NameError("NOT FOUND INFORMATION ON FRAME in the range of lines with index {}-{} -- corresponding lines: \n{}".format(index, i, xml_lines_of_sentence[index:i+1]))
            
            target_dict = {}
            target_dict[frame_name] = target_content
            frame_content.append(target_dict)
            frame_content.append(fe_dict)
            
            break
        
        elif line.startswith(utils_SALSA.STRING.FRAME_OPEN_start.value):
            match = utils_SALSA.PATTERN.ATT_name.value.search(line)
            if not match:
                raise NameError("NOT FOUND FRAME-name in: {}".format(line))
            frame_name = match.group(utils_SALSA.GROUPNAME.name.value)
            
            match = utils_SALSA.PATTERN.ATT_FRAME_id.value.search(line)
            if not match:
                raise NameError("NOT FOUND FRAME-id in: {}".format(line))
            #frame_id = int(match.group(utils_SALSA.GROUPNAME.frameId.value))
            whole_frame_id = match.group(utils_SALSA.GROUPNAME.wholeFrameId.value)
        
        elif line.startswith(utils_SALSA.STRING.TARGET_OPEN_start.value):
            is_target_open = True
        
        elif line == utils_SALSA.STRING.TARGET_CLOSE.value:
            is_target_open = False
        
        elif line.startswith(utils_SALSA.STRING.FE_OPEN_start.value):
            match = utils_SALSA.PATTERN.ATT_name.value.search(line)
            if not match:
                raise NameError("NOT FOUND FE-name in: {}".format(line))
            current_fe_name = match.group(utils_SALSA.GROUPNAME.name.value)
            
            is_fe_open = True
        
        elif line == utils_SALSA.STRING.FE_CLOSE.value:
            current_fe_name = None
            is_fe_open = False
        
        
        elif line.startswith(utils_SALSA.STRING.FENODE_EMPTY_start.value):
    
            match = utils_SALSA.PATTERN.ATT_FENODE_idref.value.search(line)
            if not match:
                #try a variant for e.g. <fenode idref="steigen_s37355_2_s0" is_split="yes"/>
                match = utils_SALSA.PATTERN.ATT_FENODE_idref_var.value.search(line)
                if not match:
                    raise NameError("NOT FOUND FENODE-idref in: {}".format(line))
            sentid = int(match.group(utils_SALSA.GROUPNAME.sentId.value))
            idref = int(match.group(utils_SALSA.GROUPNAME.fenodeIdref.value))
   
            if is_target_open:
                if is_fe_open: #only one of them should be open
                    raise NameError() #FIXME make the issue explicit
                target_content.append( (sentid, idref) )
    
            elif is_fe_open:
                
                fe_dict.setdefault(current_fe_name, [])
                fe_dict[current_fe_name].append( (sentid, idref) )
            
            else: #one of the above tags should be open
                raise NameError() #FIXME make the issue explicit
    
        else:
            if line.startswith("<flag "):
                continue
            raise NameError("NOT ASSUMED LINE in FRAME: {}".format(line)) 
    
    return (whole_frame_id, frame_content)

def read_salsa_data_per_sentence_dict(input_path_salsa, parallel_entry_list):
    salsa_corpora_path = input_path_salsa + utils_SALSA.SALSA.SUBPATH_TO_CORPORA.value
    salsa_verbal_xml_file_names = utils_SALSA.get_salsa_verbal_xml_file_names(salsa_corpora_path)
    list_of_salsa_sentids = __get_salsa_sent_ids(parallel_entry_list)
    
    # { sentid : {frameid : frame_content}  }
    sentenceid_to_frames_dict = {}
    for salsa_verbal_xml_file_name in salsa_verbal_xml_file_names:
        #print("- salsa_verbal_xml_file_name: ", salsa_verbal_xml_file_name)
        xml_sentence_dict = get_xml_part_for_sentences(salsa_corpora_path + salsa_verbal_xml_file_name, list_of_salsa_sentids)
        
        for xml_sentence_id, xml_sentence_lines in xml_sentence_dict.items():
            # { frame_id : frame_content }
            #frame content: [
            #       { target_name : [(sentid, tid), (sentid, tid), ...] }
            #       { fe_name : [(sentid, tid), ...], fe_name : [...], ... }
            # ]
            frames_for_sentence = get_frames_for_sentence_terminalized_nonoverlapping(xml_sentence_lines, xml_sentence_id)
            #print("--> {} frames for sentence {}".format(len(frames_for_sentence), xml_sentence_id))
            sentenceid_to_frames_dict.setdefault(xml_sentence_id, {})
            for frame_id, frame_content in frames_for_sentence.items():
                sentenceid_to_frames_dict[xml_sentence_id][frame_id] = frame_content
    
    #print("Collected frames for {} sentences.".format(len(sentenceid_to_frames_dict)))   
    return sentenceid_to_frames_dict

def __get_salsa_sent_ids(parallel_entry_list):
    list_of_salsa_sentids = []
    
    for entry in parallel_entry_list:
        salsa_sentid = entry[utils_conversion.MAPPING_FILE.SALSA_sentid.value]
        list_of_salsa_sentids.append(salsa_sentid)
    
    return list_of_salsa_sentids

def get_pas_annotation_list_conll_style(xml_lines_of_sentence, curr_sentenceid):
    '''
    extracts the predicate argument annotation for the current sentence
    :param xml_lines_of_sentence: a SALSA formatted sentence line list
    :return: list of the predicate argument structure annotations
    '''
    
            #target_dict[frame_name] = target_content
            #frame_content.append(target_dict)
            #frame_content.append(fe_dict)
    frames_dict = get_frames_for_sentence_terminalized_nonoverlapping(xml_lines_of_sentence, curr_sentenceid)
    
    #[{(pred_idx, label): [list_of_args_with_labels]}, {}]
    pas_dict_list = []
    
    predicates, arguments = __get_predicates_and_arguments_conll_style(frames_dict, curr_sentenceid)
    #print("predicates: ", predicates)
    for predicate_number_idx in range(len(predicates)):
        current_pas_dict = {}
        current_pas_dict[predicates[predicate_number_idx]] = arguments[predicate_number_idx]
        pas_dict_list.append(current_pas_dict)
            
    return pas_dict_list


def __get_predicates_and_arguments_conll_style(frames_dict, sentid):
    #conll_style: not id but idx: idx = id-1
    
    #frames_dict: frames_dict[whole_frame_id] = [ predicate_dict, arguments_dict ]
    predicates = []
    arguments = []
    for frame_id, frame_content_list in frames_dict.items():
        predicate_dict, arguments_dict = frame_content_list
        
        predicate_label = list(predicate_dict.keys())[0] # only one predicate
        
        predicate_tokid_list = []
        for curr_pred_sentid, curr_pred_tokid_list in predicate_dict[predicate_label].items():
            if sentid != curr_pred_sentid:
                continue
            for curr_pred_tokid in curr_pred_tokid_list:
                predicate_tokid_list.append(curr_pred_tokid-1)
        predicate_tokid_list.sort() 
        
        predicates.append( (predicate_tokid_list, predicate_label) )
        
        argument_list = []
        
        for argument_label, argument_content in arguments_dict.items():
            argument_tokid_list = []
            curr_arg_sentid, curr_arg_tokid_list = argument_content
            if curr_arg_sentid != sentid:
                continue
            for curr_arg_id in curr_arg_tokid_list:
                argument_tokid_list.append(curr_arg_id-1)
            argument_tokid_list.sort()
            argument_list.append ( (argument_tokid_list, argument_label) )
        argument_list = sorted(argument_list, key=lambda x: x[0][0])
        arguments.append(argument_list)
    
    predicates_and_arguments = list(zip(predicates, arguments))
    predicates_and_arguments = sorted(predicates_and_arguments, key=lambda x: x[0][0][0])
    
    predicates = [x[0] for x in predicates_and_arguments]
    arguments = [x[1] for x in predicates_and_arguments]
    
    return (predicates, arguments)




def main_test():
    input_path = "/Users/eva/my/uni/Data/SALSA/"
    input_file_name = input_path + "salsa-corpora-2.0/salsa-corpora/abfragen.xml"
    
    
    dict_of_xml_sentence_lines = get_xml_part_for_sentences(input_file_name)
    print("sentences: {}".format(len(dict_of_xml_sentence_lines)))
    
    for sentenceid, xml_sentence_lines in dict_of_xml_sentence_lines.items():
        print("---")
        print("SENTENCE id:{}".format(sentenceid))
        print()
        terminals_dict, nonterminals_dict = __read_tnt_graph(xml_sentence_lines)
        print("TERMINALS")
        print(terminals_dict)
        print()
        print("NONTERMINALS")
        print(nonterminals_dict)
        print()
        
        frames_dict_terminalized_nonoverlapping = get_frames_for_sentence_terminalized_nonoverlapping(xml_sentence_lines, sentenceid)
        print("FRAMES with terminals")
        print(frames_dict_terminalized_nonoverlapping)

if __name__ == "__main__":
    main_test()