#!/usr/bin/env python3.5 

'''
This work is licensed under the Creative Commons Attribution-ShareAlike 4.0 International License. To view a copy of this license, visit http://creativecommons.org/licenses/by-sa/4.0/.

Website: http://projects.cl.uni-heidelberg.de/SR3de/
Contact: Eva Mujdricza-Maydt, mujdricza@cl.uni-heidelberg.de

'''

__author__ = "mujdricza-maydt"

'''
Constant-like variables for extracting sentences in particular the predicate argument structures in the SALSA-2.0 corpus.
'''

from enum import Enum
import re, os


class STRING(Enum):
    '''constants for relevant xml tags '''
    SENTENCE_OPEN_start = "<s "
    SENTENCE_CLOSE = "</s>"
    TERMINALS_OPEN = '<terminals>'
    TERMINALS_CLOSE = '</terminals>'
    TERMINAL_EMPTY_start = "<t "
    TERMINAL_CLOSE = "</t>" #occures a few times
    ATT_id_start = "id="
    ATT_token_start = "word="
    ATT_pos_start = "pos="
    
    NONTERMINALS_OPEN = '<nonterminals>'
    NONTERMINALS_CLOSE = '</nonterminals>'
    NONTERMINAL_OPEN_start = "<nt "
    NONTERMINAL_CLOSE = "</nt>"
    
    EDGE_EMPTY_start = "<edge "
    SECEDGE_EMPTY_start = "<secedge " #occures a few times
    
    FRAMES_OPEN = "<frames>"
    FRAMES_CLOSE = "</frames>"
    FRAME_OPEN_start = "<frame "
    FRAME_CLOSE = "</frame>"
    TARGET_OPEN_start = "<target "
    TARGET_CLOSE = "</target>"
    
    FE_OPEN_start = "<fe "
    FE_CLOSE = "</fe>"
    FENODE_EMPTY_start = "<fenode "
    

class PATTERN(Enum):
    '''compiled regular expressions for relevant xml tags and attributes'''
    ATT_S_id = re.compile('id="s(?P<sentId>[0-9]+)"')

    #e.g.: <t id="s12926_1" lemma="auf" person="--" degree="--" tense="--" mood="--" gender="--" number="--" word="Auf" case="--" morph="--" pos="APPR"/>
    #TOKEN_EMPTY = '<t id="(?P<tokenId>s[0-9_]+)" .+? word="(?P<token>[^"]+?)" .+? pos="(?P<pos>[^"]+?)" .+?/?>$'
    ATT_T_id = re.compile('id="s(?P<sentId>[0-9]+)_(?P<tokenId>[0-9]+)"')
    ATT_word = re.compile('word="(?P<word>[^"]+?)"')
    ATT_pos = re.compile('pos="(?P<pos>[^"]+?)"')
    
    #nonterminal e.g.
    #<nt id="s17157_505" cat="S">
    #    <edge label="MO" idref="s17157_11"/>
    #    <edge label="MO" idref="s17157_501"/>
    #</nt>
    ATT_NT_id = re.compile('id="s(?P<sentId>[0-9]+)_(?P<ntId>[0-9]+)"')
    ATT_NT_id_VROOT = re.compile('id="s(?P<sentId>[0-9]+)_(?P<ntId>VROOT)"')
    ATT_EDGE_idref = re.compile('idref="s(?P<sentId>[0-9]+)_(?P<ntedgeIdref>[0-9]+)"')
    
    ATT_name = re.compile('name="(?P<name>[^"]+)"')
    ATT_FENODE_idref = re.compile('idref="s(?P<sentId>[0-9]+)_(?P<fenodeIdref>[0-9]+)"')
    #e.g. <fenode idref="steigen_s37355_2_s0" is_split="yes"/>
    ATT_FENODE_idref_var = re.compile('idref="([^_]+?)_s(?P<sentId>[0-9]+)_(?P<fenodeIdref>[0-9]+)_') 
    ATT_FRAME_id = re.compile('id="(?P<wholeFrameId>(?P<lemma>[^_]+)_s(?P<sentId>[0-9]+)_f(?P<frameId>[0-9]+))"')
    ATT_FE_id = re.compile('id="(?P<lemma>[^_]+)_s(?P<sentId>[0-9]+)_f(?P<frameId>[0-9]+)_e(?P<feId>[0-9]+)"')


class GROUPNAME(Enum):
    '''used group names in regular expressions'''
    sentId = "sentId" #counted from 1
    tokenId = "tokenId" #counted from 1
    ntId = "ntId" #counted from 500
    frameId = "frameId" #not unique! --> take the whole value as id!!
    wholeFrameId = "wholeFrameId"
    feId = "feId" #counted from 1
    ntedgeIdref = "ntedgeIdref" #ref to t/nt id
    fenodeIdref = "fenodeIdref" #ref to t/nt id
    word = "word"
    pos = "pos"
    name = "name"
    lemma = "lemma"

class ESCAPE(Enum):
    #special value for a root nonterminal id
    VROOT_id = "1000"

class SALSA(Enum):
    SUBPATH_TO_CORPORA = "salsa-corpora-2.0/salsa-corpora/"
    XML_FILE_EXT = ".xml"

def get_salsa_verbal_xml_file_names(salsa_corpora_path):
    salsa_verbal_xml_file_names = [fn for fn in os.listdir(salsa_corpora_path) if (fn.endswith(SALSA.XML_FILE_EXT.value) and fn == fn.lower())]
    return salsa_verbal_xml_file_names