#!/usr/bin/env python3.5 

'''
This work is licensed under the Creative Commons Attribution-ShareAlike 4.0 International License. To view a copy of this license, visit http://creativecommons.org/licenses/by-sa/4.0/.

Website: http://projects.cl.uni-heidelberg.de/SR3de/
Contact: Eva Mujdricza-Maydt, mujdricza@cl.uni-heidelberg.de

'''

__author__ = "mujdricza-maydt"

'''
Constant-like variables for creating the SR3de corpus.

'''

import os
from enum import Enum

#FIXME too complicated
class MAPPING_FILE(Enum):
    '''to access to the columns in the file mapping the parts of the corpus annotation'''
    SR3deVN_dataset = 0
    SR3deVN_sentidx = 1
    SR3deVN_tokidx = 2
    SR3deVN_predlabel = 3
    CoNLL2009_dataset = 4
    CoNLL2009_sentidx = 5
    CoNLL2009_tokidx = 6
    CoNLL2009_predlabel = 7
    SALSA_dataset = 8 #dummy
    SALSA_sentid = 9
    SALSA_tokid = 10 #dummy
    SALSA_xmlfile = 11
    
    _STR_TYPE = ["SR3deVN_dataset", "SR3deVN_predlabel", "CoNLL2009_dataset", "CoNLL2009_predlabel", "SALSA_dataset", "SALSA_xmlfile"]
    _INT_TYPE = ["SR3deVN_sentidx", "SR3deVN_tokidx", "CoNLL2009_sentidx", "CoNLL2009_tokidx", "SALSA_sentid", "SALSA_tokid"]
    
    
    def retype(idx, item):
        retyped_item = item
        found_member = False
        for member in list(MAPPING_FILE.__members__):
            if MAPPING_FILE.__members__[member].value == idx:
                member_name = MAPPING_FILE.__members__[member].name
                #print("-> found member name: " + member_name)
                
                if member_name in MAPPING_FILE.__members__["_STR_TYPE"].value:
                    found_member = True
                    retyped_item = str(retyped_item)
                elif member_name in MAPPING_FILE.__members__["_INT_TYPE"].value:
                    found_member = True
                    retyped_item = int(retyped_item)
                #print("--> found_member: {}".format(found_member))
            #else:
                #print("idx = {} <-> member value: {}, name: {}".format(idx, MAPPING_FILE.__members__[member].value, member))
        #print("after retype found:{}: idx {}, item:{} -- type:{}, retyped_item:{} -- type: {}".format(found_member, idx, item, type(item), retyped_item, type(retyped_item)))
        
        if found_member == False:
            raise IndexError("NO MAPPING_FILE member with idx value = {}".format(idx))
        
        return retyped_item

class PARALLEL_ITEM(Enum):
    ##sr3de_dataset	sr3de_sidx	sr3de_tidx	sr3de_plabel
    IDX_dataset = 0
    IDX_sentid = 1 #NOTE: "id" is index for CONLL and SR3de_VN, but the id number for SALSA
    IDX_tokid = 2
    IDX_predlabel = 3 #NOTE: label is for SALSA the file name the sentence with the given predicate is stored
    
class DATA_NAME(Enum):
    '''names for the parts of the corpus with different SR annotation '''
    CoNLL2009 = "CoNLL2009"
    SR3de_VN = "SR3de_VN"
    SALSA = "SALSA"
    

class FORMAT(Enum):
    '''commonly used variables'''
    TAB = "\t"
    NL = "\n"
    COMMENTLINE_start = "#"

class OUTPUT_PATH_PART(Enum):
    '''for the output path structure'''
    SR3de = "SR3de"
    SR3de_FN = "SR3de/FN/"
    SR3de_PB = "SR3de/PB/"
    SR3de_VN = "SR3de/VN/"

class DATASET(Enum):
    '''dataset names'''
    train = "train"
    dev = "dev"
    test = "test"

class SR3de_VN_FILES(Enum):
    '''input file names in the SR3de_VN data'''
    train = "sr3de_train.conll"
    dev = "sr3de_devresmpl.conll"
    test = "sr3de_test.conll"

class CoNLL2009_FILES(Enum):
    '''assumed names for the CoNLL2009 data files'''
    train = "CoNLL2009-ST-German-train.txt"
    dev = "CoNLL2009-ST-German-development.txt"
    test = "CoNLL2009-ST-evaluation-German.txt"

class SR3de_PARALLEL_FILES(Enum):
    '''names for the parallel output files '''
    train = "sr3de_train.conll"
    dev = "sr3de_dev.conll"
    test = "sr3de_test.conll"

def is_not_relevant_line(line):
    '''empty or commented lines are not relevant'''
    line = line.strip()
    if len(line) == 0:
        return True
    if line.startswith(FORMAT.COMMENTLINE_start.value):
        return True
    return False

def create_path_if_needed(output_path):
    os.makedirs(output_path, exist_ok=True)

def create_parallel_path_structure(path_to_the_parallel_data):
    for path_part_member_name in list(OUTPUT_PATH_PART.__members__):
        create_path_if_needed(path_to_the_parallel_data + OUTPUT_PATH_PART.__members__[path_part_member_name].value)

def read_mapping_file(input_file_name):
    f = open(input_file_name)
    lines = f.readlines()
    f.close()
    
    parallel_entry_list = []
    for line in lines:
        if is_not_relevant_line(line):
            continue
        
        line_split = line.strip().split(FORMAT.TAB.value)
        entry = []
        for i in range(len(line_split)):
            entry.append(MAPPING_FILE.retype(i, line_split[i])) 
        parallel_entry_list.append(entry)
        
    return parallel_entry_list

def get_parallel_entry_dict(parallel_entry_list, key=DATA_NAME.SR3de_VN.value):
    
    # { key_dataset : { key_sentid : [(the whole entry),  (maybe another annotation in the same sentence)], [annotations for next sentence] }  }
    entry_dict = {}
    
    if key == DATA_NAME.SR3de_VN.value:
        for entry in parallel_entry_list:
            
            k_dataset = entry[MAPPING_FILE.SR3deVN_dataset.value]
            k_sentid = entry[MAPPING_FILE.SR3deVN_sentidx.value]
            
            entry_dict.setdefault(k_dataset, {})
            entry_dict[k_dataset].setdefault(k_sentid, [])
            entry_dict[k_dataset][k_sentid].append( tuple(entry) )
    else:
        raise NameError("NOT ASSUMED DICTIONARY structure for the parallel entry dict with key={}".format(key))
    
    return entry_dict
    
    

def normalize_path(path_name):
    if not path_name.endswith(os.path.sep):
        path_name =+ os.path.sep
    return path_name

def check_list_lengths(list_1, list_2):
    '''for lists which should have the same length'''
    if len(list_1) != len(list_2):
        #raise IndexError("NOT EXPECTED UNEQUALITY IN LIST LENGTHS list_1:{} != list_2:{}".format(len(list_1), len(list_2)))
        raise IndexError("NOT EXPECTED UNEQUALITY IN LIST LENGTHS list_1:{} != list_2:{}\nLIST_1: {}\nLIST_2: {}".format(len(list_1), len(list_2), list_1, list_2))